@@ -9,18 +9,21 @@ from mypy.types import (
 )
 from mypy.nodes import (
     TypeInfo, FuncBase, Var, FuncDef, SymbolNode, Context, MypyFile, TypeVarExpr,
-    ARG_POS, ARG_STAR, ARG_STAR2, Decorator, OverloadedFuncDef, TypeAlias, TempNode
+    ARG_POS, ARG_STAR, ARG_STAR2, Decorator, OverloadedFuncDef, TypeAlias, TempNode,
+    is_final_node, SYMBOL_FUNCBASE_TYPES,
 )
 from mypy.messages import MessageBuilder
 from mypy.maptype import map_instance_to_supertype
 from mypy.expandtype import expand_type_by_instance, expand_type, freshen_function_type_vars
+from mypy.erasetype import erase_typevars
 from mypy.infer import infer_type_arguments
 from mypy.typevars import fill_typevars
 from mypy.plugin import AttributeContext
 from mypy.typeanal import set_any_tvars
-from mypy import messages
+from mypy import message_registry
 from mypy import subtypes
 from mypy import meet
+from mypy.typeops import tuple_fallback
 
 MYPY = False
 if MYPY:  # import for forward declaration only
@@ -121,7 +124,10 @@ def _analyze_member_access(name: str,
         return analyze_type_callable_member_access(name, typ, mx)
     elif isinstance(typ, TypeType):
         return analyze_type_type_member_access(name, typ, mx)
-    elif isinstance(typ, (TupleType, TypedDictType, LiteralType, FunctionLike)):
+    elif isinstance(typ, TupleType):
+        # Actually look up from the fallback instance type.
+        return _analyze_member_access(name, tuple_fallback(typ), mx)
+    elif isinstance(typ, (TypedDictType, LiteralType, FunctionLike)):
         # Actually look up from the fallback instance type.
         return _analyze_member_access(name, typ.fallback, mx)
     elif isinstance(typ, NoneTyp):
@@ -147,7 +153,7 @@ def analyze_instance_member_access(name: str,
     if name == '__init__' and not mx.is_super:
         # Accessing __init__ in statically typed code would compromise
         # type safety unless used via super().
-        mx.msg.fail(messages.CANNOT_ACCESS_INIT, mx.context)
+        mx.msg.fail(message_registry.CANNOT_ACCESS_INIT, mx.context)
         return AnyType(TypeOfAny.from_error)
 
     # The base object has an instance type.
@@ -194,7 +200,7 @@ def analyze_type_callable_member_access(name: str,
     # TODO super?
     ret_type = typ.items()[0].ret_type
     if isinstance(ret_type, TupleType):
-        ret_type = ret_type.fallback
+        ret_type = tuple_fallback(ret_type)
     if isinstance(ret_type, Instance):
         if not mx.is_operator:
             # When Python sees an operator (eg `3 == 4`), it automatically translates that
@@ -234,7 +240,7 @@ def analyze_type_type_member_access(name: str, typ: TypeType, mx: MemberContext)
         if isinstance(typ.item.upper_bound, Instance):
             item = typ.item.upper_bound
     elif isinstance(typ.item, TupleType):
-        item = typ.item.fallback
+        item = tuple_fallback(typ.item)
     elif isinstance(typ.item, FunctionLike) and typ.item.is_type_obj():
         item = typ.item.fallback
     elif isinstance(typ.item, TypeType):
@@ -339,7 +345,15 @@ def analyze_member_var_access(name: str,
                     typ = map_instance_to_supertype(itype, method.info)
                     getattr_type = expand_type_by_instance(bound_method, typ)
                     if isinstance(getattr_type, CallableType):
-                        return getattr_type.ret_type
+                        result = getattr_type.ret_type
+
+                        # Call the attribute hook before returning.
+                        fullname = '{}.{}'.format(method.info.fullname(), name)
+                        hook = mx.chk.plugin.get_attribute_hook(fullname)
+                        if hook:
+                            result = hook(AttributeContext(mx.original_type, result,
+                                                           mx.context, mx.chk))
+                        return result
         else:
             setattr_meth = info.get_method('__setattr__')
             if setattr_meth and setattr_meth.info.fullname() != 'builtins.object':
@@ -405,7 +419,7 @@ def analyze_descriptor_access(instance_type: Type,
     dunder_get = descriptor_type.type.get_method('__get__')
 
     if dunder_get is None:
-        msg.fail(messages.DESCRIPTOR_GET_NOT_CALLABLE.format(descriptor_type), context)
+        msg.fail(message_registry.DESCRIPTOR_GET_NOT_CALLABLE.format(descriptor_type), context)
         return AnyType(TypeOfAny.from_error)
 
     function = function_type(dunder_get, builtin_type('builtins.function'))
@@ -432,7 +446,7 @@ def analyze_descriptor_access(instance_type: Type,
         return inferred_dunder_get_type
 
     if not isinstance(inferred_dunder_get_type, CallableType):
-        msg.fail(messages.DESCRIPTOR_GET_NOT_CALLABLE.format(descriptor_type), context)
+        msg.fail(message_registry.DESCRIPTOR_GET_NOT_CALLABLE.format(descriptor_type), context)
         return AnyType(TypeOfAny.from_error)
 
     return inferred_dunder_get_type.ret_type
@@ -586,12 +600,12 @@ def analyze_class_attribute_access(itype: Instance,
         if is_method:
             mx.msg.cant_assign_to_method(mx.context)
         if isinstance(node.node, TypeInfo):
-            mx.msg.fail(messages.CANNOT_ASSIGN_TO_TYPE, mx.context)
+            mx.msg.fail(message_registry.CANNOT_ASSIGN_TO_TYPE, mx.context)
 
     # If a final attribute was declared on `self` in `__init__`, then it
     # can't be accessed on the class object.
     if node.implicit and isinstance(node.node, Var) and node.node.is_final:
-        mx.msg.fail(messages.CANNOT_ACCESS_FINAL_INSTANCE_ATTR
+        mx.msg.fail(message_registry.CANNOT_ACCESS_FINAL_INSTANCE_ATTR
                     .format(node.node.name()), mx.context)
 
     # An assignment to final attribute on class object is also always an error,
@@ -608,11 +622,47 @@ def analyze_class_attribute_access(itype: Instance,
             symnode = node.node
             assert isinstance(symnode, Var)
             return mx.chk.handle_partial_var_type(t, mx.is_lvalue, symnode, mx.context)
-        if not is_method and (isinstance(t, TypeVarType) or get_type_vars(t)):
-            mx.msg.fail(messages.GENERIC_INSTANCE_VAR_CLASS_ACCESS, mx.context)
+
+        # Find the class where method/variable was defined.
+        if isinstance(node.node, Decorator):
+            super_info = node.node.var.info  # type: Optional[TypeInfo]
+        elif isinstance(node.node, (Var, SYMBOL_FUNCBASE_TYPES)):
+            super_info = node.node.info
+        else:
+            super_info = None
+
+        # Map the type to how it would look as a defining class. For example:
+        #     class C(Generic[T]): ...
+        #     class D(C[Tuple[T, S]]): ...
+        #     D[int, str].method()
+        # Here itype is D[int, str], isuper is C[Tuple[int, str]].
+        if not super_info:
+            isuper = None
+        else:
+            isuper = map_instance_to_supertype(itype, super_info)
+
+        if isinstance(node.node, Var):
+            assert isuper is not None
+            # Check if original variable type has type variables. For example:
+            #     class C(Generic[T]):
+            #         x: T
+            #     C.x  # Error, ambiguous access
+            #     C[int].x  # Also an error, since C[int] is same as C at runtime
+            if isinstance(t, TypeVarType) or get_type_vars(t):
+                # Exception: access on Type[...], including first argument of class methods is OK.
+                if not isinstance(mx.original_type, TypeType):
+                    mx.msg.fail(message_registry.GENERIC_INSTANCE_VAR_CLASS_ACCESS, mx.context)
+
+            # Erase non-mapped variables, but keep mapped ones, even if there is an error.
+            # In the above example this means that we infer following types:
+            #     C.x -> Any
+            #     C[int].x -> int
+            t = erase_typevars(expand_type_by_instance(t, isuper))
+
         is_classmethod = ((is_decorated and cast(Decorator, node.node).func.is_class)
                           or (isinstance(node.node, FuncBase) and node.node.is_class))
-        result = add_class_tvars(t, itype, is_classmethod, mx.builtin_type, mx.original_type)
+        result = add_class_tvars(t, itype, isuper, is_classmethod, mx.builtin_type,
+                                 mx.original_type)
         if not mx.is_lvalue:
             result = analyze_descriptor_access(mx.original_type, result, mx.builtin_type,
                                                mx.msg, mx.context, chk=mx.chk)
@@ -622,7 +672,7 @@ def analyze_class_attribute_access(itype: Instance,
         return AnyType(TypeOfAny.special_form)
 
     if isinstance(node.node, TypeVarExpr):
-        mx.msg.fail(messages.CANNOT_USE_TYPEVAR_AS_EXPRESSION.format(
+        mx.msg.fail(message_registry.CANNOT_USE_TYPEVAR_AS_EXPRESSION.format(
                     itype.type.name(), name), mx.context)
         return AnyType(TypeOfAny.from_error)
 
@@ -647,17 +697,17 @@ def analyze_class_attribute_access(itype: Instance,
         return function_type(cast(FuncBase, node.node), mx.builtin_type('builtins.function'))
 
 
-def add_class_tvars(t: Type, itype: Instance, is_classmethod: bool,
+def add_class_tvars(t: Type, itype: Instance, isuper: Optional[Instance], is_classmethod: bool,
                     builtin_type: Callable[[str], Instance],
                     original_type: Type) -> Type:
     """Instantiate type variables during analyze_class_attribute_access,
     e.g T and Q in the following:
 
-    def A(Generic(T)):
+    class A(Generic[T]):
         @classmethod
         def foo(cls: Type[Q]) -> Tuple[T, Q]: ...
 
-    class B(A): pass
+    class B(A[str]): pass
 
     B.foo()
 
@@ -665,15 +715,36 @@ def add_class_tvars(t: Type, itype: Instance, is_classmethod: bool,
     """
     # TODO: verify consistency between Q and T
     info = itype.type  # type: TypeInfo
+    if is_classmethod:
+        assert isuper is not None
+        t = expand_type_by_instance(t, isuper)
+    # We add class type variables if the class method is accessed on class object
+    # without applied type arguments, this matches the behavior of __init__().
+    # For example (continuing the example in docstring):
+    #     A       # The type of callable is def [T] () -> A[T], _not_ def () -> A[Any]
+    #     A[int]  # The type of callable is def () -> A[int]
+    # and
+    #     A.foo       # The type is generic def [T] () -> Tuple[T, A[T]]
+    #     A[int].foo  # The type is non-generic def () -> Tuple[int, A[int]]
+    #
+    # This behaviour is useful for defining alternative constructors for generic classes.
+    # To achieve such behaviour, we add the class type variables that are still free
+    # (i.e. appear in the return type of the class object on which the method was accessed).
+    free_ids = {t.id for t in itype.args if isinstance(t, TypeVarType)}
+
     if isinstance(t, CallableType):
-        # TODO: Should we propagate type variable values?
+        # NOTE: in practice either all or none of the variables are free, since
+        # visit_type_application() will detect any type argument count mismatch and apply
+        # a correct number of Anys.
         tvars = [TypeVarDef(n, n, i + 1, [], builtin_type('builtins.object'), tv.variance)
-                 for (i, n), tv in zip(enumerate(info.type_vars), info.defn.type_vars)]
+                 for (i, n), tv in zip(enumerate(info.type_vars), info.defn.type_vars)
+                 # use 'is' to avoid id clashes with unrelated variables
+                 if any(tv.id is id for id in free_ids)]
         if is_classmethod:
             t = bind_self(t, original_type, is_classmethod=True)
         return t.copy_modified(variables=tvars + t.variables)
     elif isinstance(t, Overloaded):
-        return Overloaded([cast(CallableType, add_class_tvars(item, itype, is_classmethod,
+        return Overloaded([cast(CallableType, add_class_tvars(item, itype, isuper, is_classmethod,
                                                               builtin_type, original_type))
                            for item in t.items()])
     return t
@@ -795,7 +866,7 @@ def map_type_from_supertype(typ: Type,
     # Create the type of self in subtype, of form t[a1, ...].
     inst_type = fill_typevars(sub_info)
     if isinstance(inst_type, TupleType):
-        inst_type = inst_type.fallback
+        inst_type = tuple_fallback(inst_type)
     # Map the type of self to supertype. This gets us a description of the
     # supertype type variables in terms of subtype variables, i.e. t[t1, ...]
     # so that any type variables in tN are to be interpreted in subtype
@@ -895,8 +966,3 @@ def erase_to_bound(t: Type) -> Type:
         if isinstance(t.item, TypeVarType):
             return TypeType.make_normalized(t.item.upper_bound)
     return t
-
-
-def is_final_node(node: Optional[SymbolNode]) -> bool:
-    """Check whether `node` corresponds to a final attribute."""
-    return isinstance(node, (Var, FuncDef, OverloadedFuncDef, Decorator)) and node.is_final
