@@ -18,6 +18,7 @@ from typing import Any, Callable, Dict, Mapping, Optional, Tuple, List
 from mypy.dmypy_util import DEFAULT_STATUS_FILE, receive
 from mypy.ipc import IPCClient, IPCException
 from mypy.dmypy_os import alive, kill
+from mypy.util import check_python_version
 
 from mypy.version import __version__
 
@@ -59,6 +60,7 @@ p.add_argument('flags', metavar='FLAG', nargs='*', type=str,
 
 status_parser = p = subparsers.add_parser('status', help="Show daemon status")
 p.add_argument('-v', '--verbose', action='store_true', help="Print detailed status")
+p.add_argument('--fswatcher-dump-file', help="Collect information about the current file state")
 
 stop_parser = p = subparsers.add_parser('stop', help="Stop daemon (asks it politely to go away)")
 
@@ -95,6 +97,15 @@ p.add_argument('--update', metavar='FILE', nargs='*',
 p.add_argument('--remove', metavar='FILE', nargs='*',
                help="Files to remove from the run")
 
+suggest_parser = p = subparsers.add_parser('suggest',
+    help="Suggest a signature or show call sites for a specific function")
+p.add_argument('function', metavar='FUNCTION', type=str,
+               help="Function specified as '[package.]module.[class.]function'")
+p.add_argument('--json', action='store_true',
+               help="Produce json that pyannotate can use to apply a suggestion")
+p.add_argument('--callsites', action='store_true',
+               help="Find callsites instead of suggesting a type")
+
 hang_parser = p = subparsers.add_parser('hang', help="Hang for 100 seconds")
 
 daemon_parser = p = subparsers.add_parser('daemon', help="Run daemon in foreground")
@@ -121,6 +132,7 @@ class BadStatus(Exception):
 
 def main(argv: List[str]) -> None:
     """The code is top-down."""
+    check_python_version('dmypy')
     args = parser.parse_args(argv)
     if not args.action:
         parser.print_usage()
@@ -268,7 +280,9 @@ def do_status(args: argparse.Namespace) -> None:
     # Both check_status() and request() may raise BadStatus,
     # which will be handled by main().
     check_status(status)
-    response = request(args.status_file, 'status', timeout=5)
+    response = request(args.status_file, 'status',
+                       fswatcher_dump_file=args.fswatcher_dump_file,
+                       timeout=5)
     if args.verbose or 'error' in response:
         show_stats(response)
     if 'error' in response:
@@ -335,6 +349,18 @@ def do_recheck(args: argparse.Namespace) -> None:
     check_output(response, args.verbose, args.junit_xml, args.perf_stats_file)
 
 
+@action(suggest_parser)
+def do_suggest(args: argparse.Namespace) -> None:
+    """Ask the daemon for a suggested signature.
+
+    This just prints whatever the daemon reports as output.
+    For now it may be closer to a list of call sites.
+    """
+    response = request(args.status_file, 'suggest', function=args.function,
+                       json=args.json, callsites=args.callsites)
+    check_output(response, verbose=False, junit_xml=None, perf_stats_file=None)
+
+
 def check_output(response: Dict[str, Any], verbose: bool,
                  junit_xml: Optional[str],
                  perf_stats_file: Optional[str]) -> None:
