@@ -57,27 +57,20 @@ try:
         Attribute,
         Tuple as ast27_Tuple,
     )
-    from typed_ast import ast3
-    from typed_ast.ast3 import (
-        FunctionType,
-        Ellipsis as ast3_Ellipsis,
-    )
+    # Import ast3 from fastparse, which has special case for Python 3.8
+    from mypy.fastparse import ast3, ast3_parse
 except ImportError:
-    if sys.version_info.minor > 2:
-        try:
-            from typed_ast import ast35  # type: ignore
-        except ImportError:
-            print('The typed_ast package is not installed.\n'
-                  'You can install it with `python3 -m pip install typed-ast`.',
-                  file=sys.stderr)
-        else:
-            print('You need a more recent version of the typed_ast package.\n'
-                  'You can update to the latest version with '
-                  '`python3 -m pip install -U typed-ast`.',
-                  file=sys.stderr)
+    try:
+        from typed_ast import ast35  # type: ignore
+    except ImportError:
+        print('The typed_ast package is not installed.\n'
+              'You can install it with `python3 -m pip install typed-ast`.',
+              file=sys.stderr)
     else:
-        print('Mypy requires the typed_ast package, which is only compatible with\n'
-              'Python 3.3 and greater.', file=sys.stderr)
+        print('You need a more recent version of the typed_ast package.\n'
+              'You can update to the latest version with '
+              '`python3 -m pip install -U typed-ast`.',
+              file=sys.stderr)
     sys.exit(1)
 
 N = TypeVar('N', bound=Node)
@@ -169,6 +162,8 @@ class ASTConverter:
         # Cache of visit_X methods keyed by type of visited object
         self.visitor_cache = {}  # type: Dict[type, Callable[[Optional[AST]], Any]]
 
+        self.extra_type_ignores = []  # type: List[int]
+
     def fail(self, msg: str, line: int, column: int) -> None:
         self.errors.report(line, column, msg, blocker=True)
 
@@ -308,11 +303,12 @@ class ASTConverter:
 
     def visit_Module(self, mod: ast27.Module) -> MypyFile:
         body = self.fix_function_overloads(self.translate_stmt_list(mod.body))
-
+        ignores = [ti.lineno for ti in mod.type_ignores]
+        ignores.extend(self.extra_type_ignores)
         return MypyFile(body,
                         self.imports,
                         False,
-                        {ti.lineno for ti in mod.type_ignores},
+                        set(ignores),
                         )
 
     # --- stmt ---
@@ -339,11 +335,11 @@ class ASTConverter:
             return_type = None
         elif type_comment is not None and len(type_comment) > 0:
             try:
-                func_type_ast = ast3.parse(type_comment, '<func_type>', 'func_type')
-                assert isinstance(func_type_ast, FunctionType)
+                func_type_ast = ast3_parse(type_comment, '<func_type>', 'func_type')
+                assert isinstance(func_type_ast, ast3.FunctionType)
                 # for ellipsis arg
                 if (len(func_type_ast.argtypes) == 1 and
-                        isinstance(func_type_ast.argtypes[0], ast3_Ellipsis)):
+                        isinstance(func_type_ast.argtypes[0], ast3.Ellipsis)):
                     arg_types = [a.type_annotation
                                  if a.type_annotation is not None
                                  else AnyType(TypeOfAny.unannotated)
@@ -550,8 +546,10 @@ class ASTConverter:
     def visit_Assign(self, n: ast27.Assign) -> AssignmentStmt:
         typ = None
         if n.type_comment:
-            typ = parse_type_comment(n.type_comment, n.lineno, self.errors,
-                                     assume_str_is_unicode=self.unicode_literals)
+            extra_ignore, typ = parse_type_comment(n.type_comment, n.lineno, self.errors,
+                                                   assume_str_is_unicode=self.unicode_literals)
+            if extra_ignore:
+                self.extra_type_ignores.append(n.lineno)
 
         stmt = AssignmentStmt(self.translate_expr_list(n.targets),
                               self.visit(n.value),
@@ -568,15 +566,17 @@ class ASTConverter:
     # For(expr target, expr iter, stmt* body, stmt* orelse, string? type_comment)
     def visit_For(self, n: ast27.For) -> ForStmt:
         if n.type_comment is not None:
-            target_type = parse_type_comment(n.type_comment, n.lineno, self.errors,
-                                             assume_str_is_unicode=self.unicode_literals)
+            extra_ignore, typ = parse_type_comment(n.type_comment, n.lineno, self.errors,
+                                                   assume_str_is_unicode=self.unicode_literals)
+            if extra_ignore:
+                self.extra_type_ignores.append(n.lineno)
         else:
-            target_type = None
+            typ = None
         stmt = ForStmt(self.visit(n.target),
                        self.visit(n.iter),
                        self.as_required_block(n.body, n.lineno),
                        self.as_block(n.orelse, n.lineno),
-                       target_type)
+                       typ)
         return self.set_line(stmt, n)
 
     # While(expr test, stmt* body, stmt* orelse)
@@ -596,14 +596,16 @@ class ASTConverter:
     # With(withitem* items, stmt* body, string? type_comment)
     def visit_With(self, n: ast27.With) -> WithStmt:
         if n.type_comment is not None:
-            target_type = parse_type_comment(n.type_comment, n.lineno, self.errors,
-                                             assume_str_is_unicode=self.unicode_literals)
+            extra_ignore, typ = parse_type_comment(n.type_comment, n.lineno, self.errors,
+                                                   assume_str_is_unicode=self.unicode_literals)
+            if extra_ignore:
+                self.extra_type_ignores.append(n.lineno)
         else:
-            target_type = None
+            typ = None
         stmt = WithStmt([self.visit(n.context_expr)],
                         [self.visit(n.optional_vars)],
                         self.as_required_block(n.body, n.lineno),
-                        target_type)
+                        typ)
         return self.set_line(stmt, n)
 
     def visit_Raise(self, n: ast27.Raise) -> RaiseStmt:
