@@ -85,7 +85,7 @@ class TypeIndirectionVisitor(SyntheticTypeVisitor[Set[str]]):
         return self._visit(t.items()) | self._visit(t.fallback)
 
     def visit_tuple_type(self, t: types.TupleType) -> Set[str]:
-        return self._visit(t.items) | self._visit(t.fallback)
+        return self._visit(t.items) | self._visit(t.partial_fallback)
 
     def visit_typeddict_type(self, t: types.TypedDictType) -> Set[str]:
         return self._visit(t.items.values()) | self._visit(t.fallback)
