@@ -53,14 +53,7 @@ def main(script_path: Optional[str], args: Optional[List[str]] = None) -> None:
         args: Custom command-line arguments.  If not given, sys.argv[1:] will
         be used.
     """
-    # Check for known bad Python versions.
-    if sys.version_info[:2] < (3, 4):
-        sys.exit("Running mypy with Python 3.3 or lower is not supported; "
-                 "please upgrade to 3.4 or newer")
-    if sys.version_info[:3] == (3, 5, 0):
-        sys.exit("Running mypy with Python 3.5.0 is not supported; "
-                 "please upgrade to 3.5.1 or newer")
-
+    util.check_python_version('mypy')
     t0 = time.time()
     # To log stat() calls: os.stat = stat_proxy
     sys.setrecursionlimit(2 ** 14)
@@ -527,6 +520,11 @@ def process_options(args: List[str],
                         help="Allow unconditional variable redefinition with a new type",
                         group=strictness_group)
 
+    add_invertible_flag('--strict-equality', default=False, strict_flag=False,
+                        help="Prohibit equality, identity, and container checks for"
+                             " non-overlapping types",
+                        group=strictness_group)
+
     incremental_group = parser.add_argument_group(
         title='Incremental mode',
         description="Adjust how mypy incrementally type checks and caches modules. "
@@ -553,6 +551,9 @@ def process_options(args: List[str],
     incremental_group.add_argument(
         '--skip-version-check', action='store_true',
         help="Allow using cache written by older mypy version")
+    incremental_group.add_argument(
+        '--skip-cache-mtime-checks', action='store_true',
+        help="Skip cache internal consistency checks based on mtime")
 
     internals_group = parser.add_argument_group(
         title='Mypy internals',
@@ -612,6 +613,8 @@ def process_options(args: List[str],
 
     other_group = parser.add_argument_group(
         title='Miscellaneous')
+    other_group.add_argument(
+        '--quickstart-file', help=argparse.SUPPRESS)
     other_group.add_argument(
         '--junit-xml', help="Write junit.xml to the given file")
     other_group.add_argument(
@@ -871,6 +874,7 @@ config_types = {
     'custom_typing_module': str,
     'custom_typeshed_dir': str,
     'mypy_path': lambda s: [p.strip() for p in re.split('[,:]', s)],
+    'quickstart_file': str,
     'junit_xml': str,
     # These two are for backwards compatibility
     'silent_imports': bool,
