@@ -1,10 +1,9 @@
-"""Facilities and constants for generating error messages during type checking.
+"""Facilities for generating error messages during type checking.
 
 Don't add any non-trivial message construction logic to the type
 checker, as it can compromise clarity and make messages less
-consistent. Add such logic to this module instead. Literal messages used
-in multiple places should also be defined as constants in this module so
-they won't get out of sync.
+consistent. Add such logic to this module instead. Literal messages, including those
+with format args, should be defined as constants in mypy.message_registry.
 
 Historically we tried to avoid all message string literals in the type
 checker but we are moving away from this convention.
@@ -30,138 +29,13 @@ from mypy.nodes import (
     ReturnStmt, NameExpr, Var, CONTRAVARIANT, COVARIANT, SymbolNode,
     CallExpr
 )
+from mypy.util import unmangle
+from mypy import message_registry
 
 MYPY = False
 if MYPY:
     from typing_extensions import Final
 
-# Type checker error message constants --
-
-NO_RETURN_VALUE_EXPECTED = 'No return value expected'  # type: Final
-MISSING_RETURN_STATEMENT = 'Missing return statement'  # type: Final
-INVALID_IMPLICIT_RETURN = 'Implicit return in function which does not return'  # type: Final
-INCOMPATIBLE_RETURN_VALUE_TYPE = 'Incompatible return value type'  # type: Final
-RETURN_VALUE_EXPECTED = 'Return value expected'  # type: Final
-NO_RETURN_EXPECTED = 'Return statement in function which does not return'  # type: Final
-INVALID_EXCEPTION = 'Exception must be derived from BaseException'  # type: Final
-INVALID_EXCEPTION_TYPE = 'Exception type must be derived from BaseException'  # type: Final
-RETURN_IN_ASYNC_GENERATOR = "'return' with value in async generator is not allowed"  # type: Final
-INVALID_RETURN_TYPE_FOR_GENERATOR = \
-    'The return type of a generator function should be "Generator"' \
-    ' or one of its supertypes'  # type: Final
-INVALID_RETURN_TYPE_FOR_ASYNC_GENERATOR = \
-    'The return type of an async generator function should be "AsyncGenerator" or one of its ' \
-    'supertypes'  # type: Final
-INVALID_GENERATOR_RETURN_ITEM_TYPE = \
-    'The return type of a generator function must be None in' \
-    ' its third type parameter in Python 2'  # type: Final
-YIELD_VALUE_EXPECTED = 'Yield value expected'  # type: Final
-INCOMPATIBLE_TYPES = 'Incompatible types'  # type: Final
-INCOMPATIBLE_TYPES_IN_ASSIGNMENT = 'Incompatible types in assignment'  # type: Final
-INCOMPATIBLE_REDEFINITION = 'Incompatible redefinition'  # type: Final
-INCOMPATIBLE_TYPES_IN_AWAIT = 'Incompatible types in "await"'  # type: Final
-INCOMPATIBLE_TYPES_IN_ASYNC_WITH_AENTER = \
-    'Incompatible types in "async with" for "__aenter__"'  # type: Final
-INCOMPATIBLE_TYPES_IN_ASYNC_WITH_AEXIT = \
-    'Incompatible types in "async with" for "__aexit__"'  # type: Final
-INCOMPATIBLE_TYPES_IN_ASYNC_FOR = 'Incompatible types in "async for"'  # type: Final
-
-INCOMPATIBLE_TYPES_IN_YIELD = 'Incompatible types in "yield"'  # type: Final
-INCOMPATIBLE_TYPES_IN_YIELD_FROM = 'Incompatible types in "yield from"'  # type: Final
-INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION = \
-    'Incompatible types in string interpolation'  # type: Final
-MUST_HAVE_NONE_RETURN_TYPE = 'The return type of "{}" must be None'  # type: Final
-INVALID_TUPLE_INDEX_TYPE = 'Invalid tuple index type'  # type: Final
-TUPLE_INDEX_OUT_OF_RANGE = 'Tuple index out of range'  # type: Final
-INVALID_SLICE_INDEX = 'Slice index must be an integer or None'  # type: Final
-CANNOT_INFER_LAMBDA_TYPE = 'Cannot infer type of lambda'  # type: Final
-CANNOT_ACCESS_INIT = 'Cannot access "__init__" directly'  # type: Final
-CANNOT_ASSIGN_TO_METHOD = 'Cannot assign to a method'  # type: Final
-CANNOT_ASSIGN_TO_TYPE = 'Cannot assign to a type'  # type: Final
-INCONSISTENT_ABSTRACT_OVERLOAD = \
-    'Overloaded method has both abstract and non-abstract variants'  # type: Final
-MULTIPLE_OVERLOADS_REQUIRED = 'Single overload definition, multiple required'  # type: Final
-READ_ONLY_PROPERTY_OVERRIDES_READ_WRITE = \
-    'Read-only property cannot override read-write property'  # type: Final
-FORMAT_REQUIRES_MAPPING = 'Format requires a mapping'  # type: Final
-RETURN_TYPE_CANNOT_BE_CONTRAVARIANT = \
-    "Cannot use a contravariant type variable as return type"  # type: Final
-FUNCTION_PARAMETER_CANNOT_BE_COVARIANT = \
-    "Cannot use a covariant type variable as a parameter"  # type: Final
-INCOMPATIBLE_IMPORT_OF = "Incompatible import of"  # type: Final
-FUNCTION_TYPE_EXPECTED = "Function is missing a type annotation"  # type: Final
-ONLY_CLASS_APPLICATION = "Type application is only supported for generic classes"  # type: Final
-RETURN_TYPE_EXPECTED = "Function is missing a return type annotation"  # type: Final
-ARGUMENT_TYPE_EXPECTED = \
-    "Function is missing a type annotation for one or more arguments"  # type: Final
-KEYWORD_ARGUMENT_REQUIRES_STR_KEY_TYPE = \
-    'Keyword argument only valid with "str" key type in call to "dict"'  # type: Final
-ALL_MUST_BE_SEQ_STR = 'Type of __all__ must be {}, not {}'  # type: Final
-INVALID_TYPEDDICT_ARGS = \
-    'Expected keyword arguments, {...}, or dict(...) in TypedDict constructor'  # type: Final
-TYPEDDICT_KEY_MUST_BE_STRING_LITERAL = \
-    'Expected TypedDict key to be string literal'  # type: Final
-MALFORMED_ASSERT = 'Assertion is always true, perhaps remove parentheses?'  # type: Final
-DUPLICATE_TYPE_SIGNATURES = 'Function has duplicate type signatures'  # type: Final
-DESCRIPTOR_SET_NOT_CALLABLE = "{}.__set__ is not callable"  # type: Final
-DESCRIPTOR_GET_NOT_CALLABLE = "{}.__get__ is not callable"  # type: Final
-MODULE_LEVEL_GETATTRIBUTE = '__getattribute__ is not valid at the module level'  # type: Final
-
-# Generic
-GENERIC_INSTANCE_VAR_CLASS_ACCESS = \
-    'Access to generic instance variables via class is ambiguous'  # type: Final
-BARE_GENERIC = 'Missing type parameters for generic type'  # type: Final
-IMPLICIT_GENERIC_ANY_BUILTIN = \
-    'Implicit generic "Any". Use "{}" and specify generic parameters'  # type: Final
-
-# TypeVar
-INCOMPATIBLE_TYPEVAR_VALUE = 'Value of type variable "{}" of {} cannot be {}'  # type: Final
-CANNOT_USE_TYPEVAR_AS_EXPRESSION = \
-    'Type variable "{}.{}" cannot be used as an expression'  # type: Final
-
-# Super
-TOO_MANY_ARGS_FOR_SUPER = 'Too many arguments for "super"'  # type: Final
-TOO_FEW_ARGS_FOR_SUPER = 'Too few arguments for "super"'  # type: Final
-SUPER_WITH_SINGLE_ARG_NOT_SUPPORTED = '"super" with a single argument not supported'  # type: Final
-UNSUPPORTED_ARG_1_FOR_SUPER = 'Unsupported argument 1 for "super"'  # type: Final
-UNSUPPORTED_ARG_2_FOR_SUPER = 'Unsupported argument 2 for "super"'  # type: Final
-SUPER_VARARGS_NOT_SUPPORTED = 'Varargs not supported with "super"'  # type: Final
-SUPER_POSITIONAL_ARGS_REQUIRED = '"super" only accepts positional arguments'  # type: Final
-SUPER_ARG_2_NOT_INSTANCE_OF_ARG_1 = \
-    'Argument 2 for "super" not an instance of argument 1'  # type: Final
-SUPER_OUTSIDE_OF_METHOD_NOT_SUPPORTED = \
-    'super() outside of a method is not supported'  # type: Final
-SUPER_ENCLOSING_POSITIONAL_ARGS_REQUIRED = \
-    'super() requires one or more positional arguments in enclosing function'  # type: Final
-
-# Self-type
-MISSING_OR_INVALID_SELF_TYPE = \
-    "Self argument missing for a non-static method (or an invalid type for self)"  # type: Final
-ERASED_SELF_TYPE_NOT_SUPERTYPE = \
-    'The erased type of self "{}" is not a supertype of its class "{}"'  # type: Final
-INVALID_SELF_TYPE_OR_EXTRA_ARG = \
-    "Invalid type for self, or extra argument type in function annotation"  # type: Final
-
-# Final
-CANNOT_INHERIT_FROM_FINAL = 'Cannot inherit from final class "{}"'  # type: Final
-DEPENDENT_FINAL_IN_CLASS_BODY = \
-    "Final name declared in class body cannot depend on type variables"  # type: Final
-CANNOT_ACCESS_FINAL_INSTANCE_ATTR = \
-    'Cannot access final instance attribute "{}" on class object'  # type: Final
-
-# ClassVar
-CANNOT_OVERRIDE_INSTANCE_VAR = \
-    'Cannot override instance variable (previously declared on base class "{}") with class ' \
-    'variable'  # type: Final
-CANNOT_OVERRIDE_CLASS_VAR = \
-    'Cannot override class variable (previously declared on base class "{}") with instance ' \
-    'variable'  # type: Final
-
-# Protocol
-RUNTIME_PROTOCOL_EXPECTED = \
-    'Only @runtime protocols can be used with instance and class checks'  # type: Final
-CANNOT_INSTANTIATE_PROTOCOL = 'Cannot instantiate protocol class "{}"'  # type: Final
-
 
 ARG_CONSTRUCTOR_NAMES = {
     ARG_POS: "Arg",
@@ -334,8 +208,8 @@ class MessageBuilder:
             return typ.name
         elif isinstance(typ, TupleType):
             # Prefer the name of the fallback class (if not tuple), as it's more informative.
-            if typ.fallback.type.fullname() != 'builtins.tuple':
-                return self.format_bare(typ.fallback)
+            if typ.partial_fallback.type.fullname() != 'builtins.tuple':
+                return self.format_bare(typ.partial_fallback)
             items = []
             for t in typ.items:
                 items.append(self.format_bare(t))
@@ -647,7 +521,7 @@ class MessageBuilder:
                     msg = '{} (expression has type {}, target has type {})'
                     arg_type_str, callee_type_str = self.format_distinctly(arg_type,
                                                                            callee.arg_types[n - 1])
-                    self.fail(msg.format(INCOMPATIBLE_TYPES_IN_ASSIGNMENT,
+                    self.fail(msg.format(message_registry.INCOMPATIBLE_TYPES_IN_ASSIGNMENT,
                                          arg_type_str, callee_type_str),
                               context)
                 return
@@ -1061,7 +935,7 @@ class MessageBuilder:
                       name, base1.name(), base2.name()), context)
 
     def cant_assign_to_method(self, context: Context) -> None:
-        self.fail(CANNOT_ASSIGN_TO_METHOD, context)
+        self.fail(message_registry.CANNOT_ASSIGN_TO_METHOD, context)
 
     def cant_assign_to_classvar(self, name: str, context: Context) -> None:
         self.fail('Cannot assign to class variable "%s" via instance' % name, context)
@@ -1079,7 +953,7 @@ class MessageBuilder:
         Pass `attr_assign=True` if the assignment assigns to an attribute.
         """
         kind = "attribute" if attr_assign else "name"
-        self.fail('Cannot assign to final {} "{}"'.format(kind, name), ctx)
+        self.fail('Cannot assign to final {} "{}"'.format(kind, unmangle(name)), ctx)
 
     def protocol_members_cant_be_final(self, ctx: Context) -> None:
         self.fail("Protocol member cannot be final", ctx)
@@ -1097,11 +971,17 @@ class MessageBuilder:
                                    typ: Type,
                                    typevar_name: str,
                                    context: Context) -> None:
-        self.fail(INCOMPATIBLE_TYPEVAR_VALUE.format(typevar_name,
-                                                    callable_name(callee) or 'function',
-                                                    self.format(typ)),
+        self.fail(message_registry.INCOMPATIBLE_TYPEVAR_VALUE
+                  .format(typevar_name, callable_name(callee) or 'function', self.format(typ)),
                   context)
 
+    def dangerous_comparison(self, left: Type, right: Type, kind: str, ctx: Context) -> None:
+        left_str = 'element' if kind == 'container' else 'left operand'
+        right_str = 'container item' if kind == 'container' else 'right operand'
+        message = 'Non-overlapping {} check ({} type: {}, {} type: {})'
+        left_typ, right_typ = self.format_distinctly(left, right)
+        self.fail(message.format(kind, left_str, left_typ, right_str, right_typ), ctx)
+
     def overload_inconsistently_applies_decorator(self, decorator: str, context: Context) -> None:
         self.fail(
             'Overload does not consistently use the "@{}" '.format(decorator)
@@ -1192,7 +1072,7 @@ class MessageBuilder:
                   ctx)
 
     def need_annotation_for_var(self, node: SymbolNode, context: Context) -> None:
-        self.fail("Need type annotation for '{}'".format(node.name()), context)
+        self.fail("Need type annotation for '{}'".format(unmangle(node.name())), context)
 
     def explicit_any(self, ctx: Context) -> None:
         self.fail('Explicit "Any" is not allowed', ctx)
@@ -1345,7 +1225,11 @@ class MessageBuilder:
             # This will be only confusing a user even more.
             return
 
-        if isinstance(subtype, (TupleType, TypedDictType)):
+        if isinstance(subtype, TupleType):
+            if not isinstance(subtype.partial_fallback, Instance):
+                return
+            subtype = subtype.partial_fallback
+        elif isinstance(subtype, TypedDictType):
             if not isinstance(subtype.fallback, Instance):
                 return
             subtype = subtype.fallback
