@@ -16,15 +16,16 @@ from mypy.types import (
     CallableType, NoneTyp, DeletedType, TypeList, TypeVarDef, TypeVisitor, SyntheticTypeVisitor,
     StarType, PartialType, EllipsisType, UninhabitedType, TypeType, get_typ_args, set_typ_args,
     CallableArgument, get_type_vars, TypeQuery, union_items, TypeOfAny, ForwardRef, Overloaded,
-    LiteralType, RawExpressionType,
+    LiteralType, RawExpressionType, PlaceholderType
 )
 
 from mypy.nodes import (
-    UNBOUND_IMPORTED, TypeInfo, Context, SymbolTableNode, Var, Expression,
+    TypeInfo, Context, SymbolTableNode, Var, Expression,
     IndexExpr, RefExpr, nongen_builtins, check_arg_names, check_arg_kinds, ARG_POS, ARG_NAMED,
-    ARG_OPT, ARG_NAMED_OPT, ARG_STAR, ARG_STAR2, TypeVarExpr, FuncDef, CallExpr, NameExpr,
-    Decorator, ImportedName, TypeAlias, MypyFile
+    ARG_OPT, ARG_NAMED_OPT, ARG_STAR, ARG_STAR2, TypeVarExpr, CallExpr, NameExpr,
+    TypeAlias, PlaceholderNode
 )
+from mypy.typetraverser import TypeTraverserVisitor
 from mypy.tvar_scope import TypeVarScope
 from mypy.exprtotype import expr_to_unanalyzed_type, TypeTranslationError
 from mypy.plugin import Plugin, TypeAnalyzerPluginInterface, AnalyzeTypeContext
@@ -65,6 +66,7 @@ def analyze_type_alias(node: Expression,
                        options: Options,
                        is_typeshed_stub: bool,
                        allow_unnormalized: bool = False,
+                       allow_placeholder: bool = False,
                        in_dynamic_func: bool = False,
                        global_scope: bool = True) -> Optional[Tuple[Type, Set[str]]]:
     """Analyze r.h.s. of a (potential) type alias definition.
@@ -73,55 +75,14 @@ def analyze_type_alias(node: Expression,
     full names of type aliases it depends on (directly or indirectly).
     Return None otherwise. 'node' must have been semantically analyzed.
     """
-    # Quickly return None if the expression doesn't look like a type. Note
-    # that we don't support straight string literals as type aliases
-    # (only string literals within index expressions).
-    if isinstance(node, RefExpr):
-        # Note that this misses the case where someone tried to use a
-        # class-referenced type variable as a type alias.  It's easier to catch
-        # that one in checkmember.py
-        if isinstance(node.node, TypeVarExpr):
-            api.fail('Type variable "{}" is invalid as target for type alias'.format(
-                node.fullname), node)
-            return None
-        if not (isinstance(node.node, TypeInfo) or
-                node.fullname in ('typing.Any', 'typing.Tuple', 'typing.Callable') or
-                isinstance(node.node, TypeAlias)):
-            return None
-    elif isinstance(node, IndexExpr):
-        base = node.base
-        if isinstance(base, RefExpr):
-            if not (isinstance(base.node, TypeInfo) or
-                    base.fullname in type_constructors or
-                    isinstance(base.node, TypeAlias)):
-                return None
-            # Enums can't be generic, and without this check we may incorrectly interpret indexing
-            # an Enum class as creating a type alias.
-            if isinstance(base.node, TypeInfo) and base.node.is_enum:
-                return None
-        else:
-            return None
-    elif isinstance(node, CallExpr):
-        if (isinstance(node.callee, NameExpr) and len(node.args) == 1 and
-                isinstance(node.args[0], NameExpr)):
-            call = api.lookup_qualified(node.callee.name, node.callee)
-            arg = api.lookup_qualified(node.args[0].name, node.args[0])
-            if (call is not None and call.node and call.node.fullname() == 'builtins.type' and
-                    arg is not None and arg.node and arg.node.fullname() == 'builtins.None'):
-                return NoneTyp(), set()
-            return None
-        return None
-    else:
-        return None
-
-    # It's a type alias (though it may be an invalid one).
     try:
         type = expr_to_unanalyzed_type(node)
     except TypeTranslationError:
         api.fail('Invalid type alias', node)
         return None
     analyzer = TypeAnalyser(api, tvar_scope, plugin, options, is_typeshed_stub,
-                            allow_unnormalized=allow_unnormalized, defining_alias=True)
+                            allow_unnormalized=allow_unnormalized, defining_alias=True,
+                            allow_placeholder=allow_placeholder)
     analyzer.in_dynamic_func = in_dynamic_func
     analyzer.global_scope = global_scope
     res = type.accept(analyzer)
@@ -157,10 +118,11 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
                  allow_tuple_literal: bool = False,
                  allow_unnormalized: bool = False,
                  allow_unbound_tvars: bool = False,
+                 allow_placeholder: bool = False,
                  report_invalid_types: bool = True,
                  third_pass: bool = False) -> None:
         self.api = api
-        self.lookup = api.lookup_qualified
+        self.lookup_qualified = api.lookup_qualified
         self.lookup_fqn_func = api.lookup_fully_qualified
         self.fail_func = api.fail
         self.note_func = api.note
@@ -175,6 +137,8 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
         self.allow_unnormalized = allow_unnormalized
         # Should we accept unbound type variables (always OK in aliases)?
         self.allow_unbound_tvars = allow_unbound_tvars or defining_alias
+        # If false, record incomplete ref if we generate PlaceholderType.
+        self.allow_placeholder = allow_placeholder
         # Should we report an error whenever we encounter a RawExpressionType outside
         # of a Literal context: e.g. whenever we encounter an invalid type? Normally,
         # we want to report an error, but the caller may want to do more specialized
@@ -196,25 +160,23 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
         return typ
 
     def visit_unbound_type_nonoptional(self, t: UnboundType) -> Type:
-        sym = self.lookup(t.name, t, suppress_errors=self.third_pass)
-        if '.' in t.name:
-            # Handle indirect references to imported names.
-            #
-            # TODO: Do this for module-local references as well and remove ImportedName
-            #    type check below.
-            sym = self.api.dereference_module_cross_ref(sym)
+        sym = self.lookup_qualified(t.name, t, suppress_errors=self.third_pass)
         if sym is not None:
             node = sym.node
-            if isinstance(node, ImportedName):
-                # Forward reference to an imported name that hasn't been processed yet.
-                # To maintain backward compatibility, these get translated to Any.
-                #
-                # TODO: Remove this special case.
-                return AnyType(TypeOfAny.implementation_artifact)
+            if isinstance(node, PlaceholderNode):
+                if node.becomes_typeinfo:
+                    # Reference to placeholder type.
+                    if self.allow_placeholder:
+                        self.api.defer()
+                    else:
+                        self.api.record_incomplete_ref()
+                    return PlaceholderType(node.fullname(), self.anal_array(t.args), t.line)
+                else:
+                    # Reference to an unknown placeholder node.
+                    self.api.record_incomplete_ref()
+                    return AnyType(TypeOfAny.special_form)
             if node is None:
-                # UNBOUND_IMPORTED can happen if an unknown name was imported.
-                if sym.kind != UNBOUND_IMPORTED:
-                    self.fail('Internal error (node is None, kind={})'.format(sym.kind), t)
+                self.fail('Internal error (node is None, kind={})'.format(sym.kind), t)
                 return AnyType(TypeOfAny.special_form)
             fullname = node.fullname()
             hook = self.plugin.get_type_analyze_hook(fullname)
@@ -245,9 +207,15 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
                 all_vars = node.alias_tvars
                 target = node.target
                 an_args = self.anal_array(t.args)
-                return expand_type_alias(target, all_vars, an_args, self.fail, node.no_args, t)
+                res = expand_type_alias(target, all_vars, an_args, self.fail, node.no_args, t)
+                # The only case where expand_type_alias() can return an incorrect instance is
+                # when it is top-level instance, so no need to recurse.
+                if (isinstance(res, Instance) and len(res.args) != len(res.type.type_vars) and
+                        not self.defining_alias):
+                    fix_instance(res, self.fail)
+                return res
             elif isinstance(node, TypeInfo):
-                return self.analyze_unbound_type_with_type_info(t, node)
+                return self.analyze_type_with_type_info(node, t.args, t)
             else:
                 return self.analyze_unbound_type_without_type_info(t, sym)
         else:  # sym is None
@@ -270,6 +238,15 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
                       " in a variable annotation", t)
             return AnyType(TypeOfAny.from_error)
         elif fullname == 'typing.Tuple':
+            # Tuple is special because it is involved in builtin import cycle
+            # and may be not ready when used.
+            sym = self.api.lookup_fully_qualified_or_none('builtins.tuple')
+            if not sym or isinstance(sym.node, PlaceholderNode):
+                if self.api.is_incomplete_namespace('builtins'):
+                    self.api.record_incomplete_ref()
+                else:
+                    self.fail("Name 'tuple' is not defined", t)
+                return AnyType(TypeOfAny.special_form)
             if len(t.args) == 0 and not t.empty_tuple_index:
                 # Bare 'Tuple' is same as 'tuple'
                 if self.options.disallow_any_generics and not self.is_typeshed_stub:
@@ -320,42 +297,42 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
             return self.analyze_literal_type(t)
         return None
 
-    def analyze_unbound_type_with_type_info(self, t: UnboundType, info: TypeInfo) -> Type:
+    def analyze_type_with_type_info(self, info: TypeInfo, args: List[Type], ctx: Context) -> Type:
         """Bind unbound type when were able to find target TypeInfo.
 
         This handles simple cases like 'int', 'modname.UserClass[str]', etc.
         """
-        if len(t.args) > 0 and info.fullname() == 'builtins.tuple':
-            fallback = Instance(info, [AnyType(TypeOfAny.special_form)], t.line)
-            return TupleType(self.anal_array(t.args), fallback, t.line)
+        if len(args) > 0 and info.fullname() == 'builtins.tuple':
+            fallback = Instance(info, [AnyType(TypeOfAny.special_form)], ctx.line)
+            return TupleType(self.anal_array(args), fallback, ctx.line)
         # Analyze arguments and (usually) construct Instance type. The
         # number of type arguments and their values are
         # checked only later, since we do not always know the
         # valid count at this point. Thus we may construct an
         # Instance with an invalid number of type arguments.
-        instance = Instance(info, self.anal_array(t.args), t.line, t.column)
+        instance = Instance(info, self.anal_array(args), ctx.line, ctx.column)
         # Check type argument count.
-        if len(instance.args) != len(info.type_vars):
+        if len(instance.args) != len(info.type_vars) and not self.defining_alias:
             fix_instance(instance, self.fail)
-        if not t.args and self.options.disallow_any_generics and not self.defining_alias:
+        if not args and self.options.disallow_any_generics and not self.defining_alias:
             # We report/patch invalid built-in instances already during second pass.
             # This is done to avoid storing additional state on instances.
             # All other (including user defined) generics will be patched/reported
             # in the third pass.
             if not self.is_typeshed_stub and info.fullname() in nongen_builtins:
                 alternative = nongen_builtins[info.fullname()]
-                self.fail(message_registry.IMPLICIT_GENERIC_ANY_BUILTIN.format(alternative), t)
-                any_type = AnyType(TypeOfAny.from_error, line=t.line)
+                self.fail(message_registry.IMPLICIT_GENERIC_ANY_BUILTIN.format(alternative), ctx)
+                any_type = AnyType(TypeOfAny.from_error, line=ctx.line)
             else:
-                any_type = AnyType(TypeOfAny.from_omitted_generics, line=t.line)
+                any_type = AnyType(TypeOfAny.from_omitted_generics, line=ctx.line)
             instance.args = [any_type] * len(info.type_vars)
 
         tup = info.tuple_type
         if tup is not None:
             # The class has a Tuple[...] base class so it will be
             # represented as a tuple type.
-            if t.args:
-                self.fail('Generic tuple types not supported', t)
+            if args:
+                self.fail('Generic tuple types not supported', ctx)
                 return AnyType(TypeOfAny.from_error)
             return tup.copy_modified(items=self.anal_array(tup.items),
                                      fallback=instance)
@@ -363,8 +340,8 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
         if td is not None:
             # The class has a TypedDict[...] base class so it will be
             # represented as a typeddict type.
-            if t.args:
-                self.fail('Generic TypedDict types not supported', t)
+            if args:
+                self.fail('Generic TypedDict types not supported', ctx)
                 return AnyType(TypeOfAny.from_error)
             # Create a named TypedDictType
             return td.copy_modified(item_types=self.anal_array(list(td.items.values())),
@@ -395,26 +372,10 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
                         (not self.tvar_scope or self.tvar_scope.get_binding(sym) is None))
         if self.allow_unbound_tvars and unbound_tvar and not self.third_pass:
             return t
-        # Option 3:
-        # If it is not something clearly bad (like a known function, variable,
-        # type variable, or module), and it is still not too late, we try deferring
-        # this type using a forward reference wrapper. It will be revisited in
-        # the third pass.
-        allow_forward_ref = not (self.third_pass or
-                                 isinstance(sym.node, (FuncDef, Decorator, MypyFile,
-                                                       TypeVarExpr)) or
-                                 (isinstance(sym.node, Var) and sym.node.is_ready))
-        if allow_forward_ref:
-            # We currently can't support subscripted forward refs in functions;
-            # see https://github.com/python/mypy/pull/3952#discussion_r139950690
-            # for discussion.
-            if t.args and not self.global_scope:
-                if not self.in_dynamic_func:
-                    self.fail('Unsupported forward reference to "{}"'.format(t.name), t)
-                return AnyType(TypeOfAny.from_error)
-            return ForwardRef(t)
+
         # None of the above options worked, we give up.
         self.fail('Invalid type "{}"'.format(name), t)
+
         if self.third_pass and isinstance(sym.node, TypeVarExpr):
             self.note_func("Forward references to type variables are prohibited", t)
             return AnyType(TypeOfAny.from_error)
@@ -486,7 +447,7 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
                 return AnyType(TypeOfAny.from_error)
         any_type = AnyType(TypeOfAny.special_form)
         # If the fallback isn't filled in yet, its type will be the falsey FakeInfo
-        fallback = (t.fallback if t.fallback.type
+        fallback = (t.partial_fallback if t.partial_fallback.type
                     else self.named_type('builtins.tuple', [any_type]))
         return TupleType(self.anal_array(t.items), fallback, t.line)
 
@@ -552,6 +513,16 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
     def visit_forwardref_type(self, t: ForwardRef) -> Type:
         return t
 
+    def visit_placeholder_type(self, t: PlaceholderType) -> Type:
+        n = self.api.lookup_fully_qualified(t.fullname)
+        if isinstance(n.node, PlaceholderNode):
+            self.api.defer()  # Still incomplete
+            return t
+        else:
+            # TODO: Handle non-TypeInfo
+            assert isinstance(n.node, TypeInfo)
+            return self.analyze_type_with_type_info(n.node, t.args, t)
+
     def analyze_callable_type(self, t: UnboundType) -> Type:
         fallback = self.named_type('builtins.function')
         if len(t.args) == 0:
@@ -607,7 +578,7 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
                 names.append(arg.name)
                 if arg.constructor is None:
                     return None
-                found = self.lookup(arg.constructor, arg)
+                found = self.lookup_qualified(arg.constructor, arg)
                 if found is None:
                     # Looking it up already put an error message in
                     return None
@@ -675,7 +646,7 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
             # We report an error in only the first two cases. In the third case, we assume
             # some other region of the code has already reported a more relevant error.
             #
-            # TODO: Once we start adding support for enums, make sure we reprt a custom
+            # TODO: Once we start adding support for enums, make sure we report a custom
             # error for case 2 as well.
             if arg.type_of_any != TypeOfAny.from_error:
                 self.fail('Parameter {} of Literal[...] cannot be of type "Any"'.format(idx), ctx)
@@ -741,7 +712,8 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
         names = []  # type: List[str]
         tvars = []  # type: List[TypeVarExpr]
         for arg in type.arg_types:
-            for name, tvar_expr in arg.accept(TypeVariableQuery(self.lookup, self.tvar_scope)):
+            for name, tvar_expr in arg.accept(TypeVariableQuery(self.lookup_qualified,
+                                                                self.tvar_scope)):
                 if name not in names:
                     names.append(name)
                     tvars.append(tvar_expr)
@@ -750,7 +722,8 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
         # functions in the return type belong to those functions, not the
         # function we're currently analyzing.
         for name, tvar_expr in type.ret_type.accept(
-                TypeVariableQuery(self.lookup, self.tvar_scope, include_callables=False)):
+                TypeVariableQuery(self.lookup_qualified, self.tvar_scope,
+                                  include_callables=False)):
             if name not in names:
                 names.append(name)
                 tvars.append(tvar_expr)
@@ -763,7 +736,7 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
             return []  # We are in third pass, nothing new here
         if fun_type.variables:
             for var in fun_type.variables:
-                var_node = self.lookup(var.name, var)
+                var_node = self.lookup_qualified(var.name, var)
                 assert var_node, "Binding for function type variable not found within function"
                 var_expr = var_node.node
                 assert isinstance(var_expr, TypeVarExpr)
@@ -787,7 +760,7 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
     def is_defined_type_var(self, tvar: str, context: Context) -> bool:
         if self.tvar_scope is None:
             return False
-        tvar_node = self.lookup(tvar, context)
+        tvar_node = self.lookup_qualified(tvar, context)
         if not tvar_node:
             return False
         return self.tvar_scope.get_binding(tvar_node) is not None
@@ -1268,3 +1241,22 @@ def make_optional_type(t: Type) -> Type:
         return UnionType(items + [NoneTyp()], t.line, t.column)
     else:
         return UnionType([t, NoneTyp()], t.line, t.column)
+
+
+def fix_instance_types(t: Type, fail: Callable[[str, Context], None]) -> None:
+    """Recursively fix all instance types (type argument count) in a given type.
+
+    For example 'Union[Dict, List[str, int]]' will be transformed into
+    'Union[Dict[Any, Any], List[Any]]' in place.
+    """
+    t.accept(InstanceFixer(fail))
+
+
+class InstanceFixer(TypeTraverserVisitor):
+    def __init__(self, fail: Callable[[str, Context], None]) -> None:
+        self.fail = fail
+
+    def visit_instance(self, typ: Instance) -> None:
+        super().visit_instance(typ)
+        if len(typ.args) != len(typ.type.type_vars):
+            fix_instance(typ, self.fail)
