@@ -22,6 +22,7 @@ PER_MODULE_OPTIONS = {
     # Please keep this list sorted
     "allow_untyped_globals",
     "allow_redefinition",
+    "strict_equality",
     "always_false",
     "always_true",
     "check_untyped_defs",
@@ -157,6 +158,10 @@ class Options:
         # and the same nesting level as the initialization
         self.allow_redefinition = False
 
+        # Prohibit equality, identity, and container checks for non-overlapping types.
+        # This makes 1 == '1', 1 in ['1'], and 1 is '1' errors.
+        self.strict_equality = False
+
         # Variable names considered True
         self.always_true = []  # type: List[str]
 
@@ -169,6 +174,11 @@ class Options:
         # Config file name
         self.config_file = None  # type: Optional[str]
 
+        # A filename containing a JSON mapping from filenames to
+        # mtime/size/hash arrays, used to avoid having to recalculate
+        # source hashes as often.
+        self.quickstart_file = None  # type: Optional[str]
+
         # Write junit.xml to given file
         self.junit_xml = None  # type: Optional[str]
 
@@ -178,6 +188,7 @@ class Options:
         self.sqlite_cache = False
         self.debug_cache = False
         self.skip_version_check = False
+        self.skip_cache_mtime_checks = False
         self.fine_grained_incremental = False
         # Include fine-grained dependencies in written cache files
         self.cache_fine_grained = False
