@@ -43,10 +43,15 @@ def assert_string_arrays_equal(expected: List[str], actual: List[str],
                                msg: str) -> None:
     """Assert that two string arrays are equal.
 
+    We consider "can't" and "cannot" equivalent, by replacing the
+    former with the latter before comparing.
+
     Display any differences in a human-readable form.
     """
 
     actual = clean_up(actual)
+    actual = [line.replace("can't", "cannot") for line in actual]
+    expected = [line.replace("can't", "cannot") for line in expected]
 
     if actual != expected:
         num_skip_start = num_skipped_prefix_lines(expected, actual)
@@ -125,6 +130,20 @@ def assert_module_equivalence(name: str,
                  name))
 
 
+def assert_target_equivalence(name: str,
+                              expected: Optional[List[str]], actual: List[str]) -> None:
+    """Compare actual and expected targets (order sensitive)."""
+    if expected is not None:
+        assert_string_arrays_equal(
+            expected,
+            actual,
+            ('Actual targets ({}) do not match expected targets ({}) '
+             'for "[{} ...]"').format(
+                 ', '.join(actual),
+                 ', '.join(expected),
+                 name))
+
+
 def update_testcase_output(testcase: DataDrivenTestCase, output: List[str]) -> None:
     assert testcase.old_cwd is not None, "test was not properly set up"
     testcase_path = os.path.join(testcase.old_cwd, testcase.file)
@@ -362,6 +381,10 @@ def parse_options(program_text: str, testcase: DataDrivenTestCase,
     if testcase.config.getoption('--mypy-verbose'):
         options.verbosity = testcase.config.getoption('--mypy-verbose')
 
+    if os.getenv('NEWSEMANAL'):
+        if not flag_list or '--no-new-semantic-analyzer' not in flag_list:
+            options.new_semantic_analyzer = True
+
     return options
 
 
@@ -374,24 +397,6 @@ def split_lines(*streams: bytes) -> List[str]:
     ]
 
 
-def run_command(cmdline: List[str], *, env: Optional[Dict[str, str]] = None,
-                timeout: int = 300, cwd: str = test_temp_dir) -> Tuple[int, List[str]]:
-    """A poor man's subprocess.run() for 3.4 compatibility."""
-    process = subprocess.Popen(
-        cmdline,
-        env=env,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
-        cwd=cwd,
-    )
-    try:
-        out, err = process.communicate(timeout=timeout)
-    except subprocess.TimeoutExpired:
-        out = err = b''
-        process.kill()
-    return process.returncode, split_lines(out, err)
-
-
 def copy_and_fudge_mtime(source_path: str, target_path: str) -> None:
     # In some systems, mtime has a resolution of 1 second which can
     # cause annoying-to-debug issues when a file has the same size
