@@ -80,7 +80,9 @@ typecheck_files = [
     'check-ctypes.test',
     'check-dataclasses.test',
     'check-final.test',
+    'check-redefine.test',
     'check-literal.test',
+    'check-newsemanal.test',
 ]
 
 
@@ -148,6 +150,8 @@ class TypeCheckSuite(DataSuite):
         options.show_traceback = True
         if 'optional' in testcase.file:
             options.strict_optional = True
+        if 'newsemanal' in testcase.file:
+            options.new_semantic_analyzer = True
         if incremental_step and options.incremental:
             # Don't overwrite # flags: --no-incremental in incremental test cases
             options.incremental = True
