@@ -15,7 +15,7 @@ on specified sources.
 import os
 import re
 
-from typing import List, cast
+from typing import List, Dict, Any, Tuple, cast
 
 from mypy import build
 from mypy.modulefinder import BuildSource
@@ -27,8 +27,10 @@ from mypy.test.data import (
 )
 from mypy.test.helpers import (
     assert_string_arrays_equal, parse_options, copy_and_fudge_mtime, assert_module_equivalence,
+    assert_target_equivalence
 )
 from mypy.server.mergecheck import check_consistency
+from mypy.dmypy_util import DEFAULT_STATUS_FILE
 from mypy.dmypy_server import Server
 from mypy.main import parse_config_file
 from mypy.find_sources import create_source_list
@@ -45,6 +47,7 @@ class FineGrainedSuite(DataSuite):
         'fine-grained-cycles.test',
         'fine-grained-blockers.test',
         'fine-grained-modules.test',
+        'fine-grained-suggest.test',
     ]
     # Whether to use the fine-grained cache in the testing. This is overridden
     # by a trivial subclass to produce a suite that uses the cache.
@@ -74,12 +77,12 @@ class FineGrainedSuite(DataSuite):
 
         main_src = '\n'.join(testcase.input)
         main_path = os.path.join(test_temp_dir, 'main')
-        with open(main_path, 'w') as f:
+        with open(main_path, 'w', encoding='utf8') as f:
             f.write(main_src)
 
         options = self.get_options(main_src, testcase, build_cache=False)
         build_options = self.get_options(main_src, testcase, build_cache=True)
-        server = Server(options)
+        server = Server(options, DEFAULT_STATUS_FILE)
 
         num_regular_incremental_steps = self.get_build_steps(main_src)
         step = 1
@@ -93,6 +96,8 @@ class FineGrainedSuite(DataSuite):
         if messages:
             a.extend(normalize_messages(messages))
 
+        a.extend(self.maybe_suggest(step, server, main_src))
+
         if server.fine_grained_manager:
             if CHECK_CONSISTENCY:
                 check_consistency(server.fine_grained_manager)
@@ -118,6 +123,7 @@ class FineGrainedSuite(DataSuite):
 
             updated = []  # type: List[str]
             changed = []  # type: List[str]
+            targets = []  # type: List[str]
             if server.fine_grained_manager:
                 if CHECK_CONSISTENCY:
                     check_consistency(server.fine_grained_manager)
@@ -125,6 +131,7 @@ class FineGrainedSuite(DataSuite):
 
                 updated = server.fine_grained_manager.updated_modules
                 changed = [mod for mod, file in server.fine_grained_manager.changed_modules]
+                targets = server.fine_grained_manager.processed_targets
 
             assert_module_equivalence(
                 'stale' + str(step - 1),
@@ -134,11 +141,16 @@ class FineGrainedSuite(DataSuite):
                 'rechecked' + str(step - 1),
                 testcase.expected_rechecked_modules.get(step - 1),
                 updated)
+            assert_target_equivalence(
+                'targets' + str(step),
+                testcase.expected_fine_grained_targets.get(step),
+                targets)
 
             new_messages = normalize_messages(new_messages)
 
             a.append('==')
             a.extend(new_messages)
+            a.extend(self.maybe_suggest(step, server, main_src))
 
         # Normalize paths in test output (for Windows).
         a = [line.replace('\\', '/') for line in a]
@@ -250,6 +262,24 @@ class FineGrainedSuite(DataSuite):
             return [base] + create_source_list([test_temp_dir], options,
                                                allow_empty_dir=True)
 
+    def maybe_suggest(self, step: int, server: Server, src: str) -> List[str]:
+        output = []  # type: List[str]
+        targets = self.get_suggest(src, step)
+        for flag, target in targets:
+            json = flag.strip() == '--json'
+            callsites = flag.strip() == '--callsites'
+            res = cast(Dict[str, Any], server.cmd_suggest(target.strip(), json, callsites))
+            val = res['error'] if 'error' in res else res['out'] + res['err']
+            output.extend(val.strip().split('\n'))
+        return normalize_messages(output)
+
+    def get_suggest(self, program_text: str,
+                    incremental_step: int) -> List[Tuple[str, str]]:
+        step_bit = '1?' if incremental_step == 1 else str(incremental_step)
+        regex = '# suggest{}: (--[a-zA-Z0-9_./?^ ]+ )?([a-zA-Z0-9_./?^ ]+)$'.format(step_bit)
+        m = re.findall(regex, program_text, flags=re.MULTILINE)
+        return m
+
 
 def normalize_messages(messages: List[str]) -> List[str]:
     return [re.sub('^tmp' + re.escape(os.sep), '', message)
